@@ -79,8 +79,8 @@ sub mouthIsOpen {
 # Corresponds (I think) to TeX's \endinput
 sub flushMouth {
   my ($self) = @_;
-  $$self{mouth}->finish;;    # but not close!
-  $$self{pushback}  = [];    # And don't read anytyhing more from it.
+  $$self{mouth}->finish;    # but not close!
+  $$self{pushback}  = [];   # And don't read anytyhing more from it.
   $$self{autoclose} = 1;
   return; }
 
@@ -117,7 +117,7 @@ sub readingFromMouth {
       $self->closeMouth(1); last; }
     elsif (!@{ $$self{mouthstack} }) {
       Error('unexpected', '<closed>', $self, "Mouth is unexpectedly already closed",
-        "Reading from " . Stringify($mouth) . ", but it has already been closed."); }
+        "Reading from " . Stringify($mouth) . ", but it has already been closed."); last; }
     elsif (!$$self{autoclose} || @{ $$self{pushback} } || $$self{mouth}->hasMoreInput) {
       my $next = Stringify($self->readToken);
       Error('unexpected', $next, $self, "Unexpected input remaining: '$next'",
@@ -131,8 +131,8 @@ sub readingFromMouth {
 # User feedback for where something (error?) occurred.
 sub getLocator {
   my ($self) = @_;
-  my $mouth = $$self{mouth};
-  my $i     = 0;
+  my $mouth  = $$self{mouth};
+  my $i      = 0;
   while ((defined $mouth) && (!defined $$mouth{source})
     && ($i < scalar(@{ $$self{mouthstack} }))) {
     $mouth = $$self{mouthstack}[$i++][0]; }
@@ -140,7 +140,8 @@ sub getLocator {
   return $loc if defined $loc;
   foreach my $frame (@{ $$self{mouthstack} }) {
     my $ml = $$frame[0]->getLocator;
-    return $ml if defined $ml; } }
+    return $ml if defined $ml; }
+  return; }
 
 sub getSource {
   my ($self) = @_;
@@ -281,7 +282,7 @@ sub readRawLine {
   my ($self) = @_;
   # If we've got unread tokens, they presumably should come before the Mouth's raw data
   # but we'll convert them back to string.
-  my @tokens = @{ $$self{pushback} };
+  my @tokens  = @{ $$self{pushback} };
   my @markers = grep { $_->getCatcode == CC_MARKER } @tokens;
   if (@markers) {    # Whoops, profiling markers!
     @tokens = grep { $_->getCatcode != CC_MARKER } @tokens;    # Remove
@@ -618,7 +619,7 @@ sub readNormalInteger {
     return Number(hex($self->readDigits('0-9A-F', 1))); }
   elsif ($token->equals(T_OTHER("\`"))) {                                   # Read Charcode
     my $next = $self->readToken;
-    my $s = ($next && $next->getString) || '';
+    my $s    = ($next && $next->getString) || '';
     $s =~ s/^\\//;
     return Number(ord($s)); }    # Only a character token!!! NOT expanded!!!!
   else {
@@ -695,7 +696,8 @@ sub readUnit {
     return $u->valueOf; }
   else {
     $self->readKeyword('true');    # But ignore, we're not bothering with mag...
-    $u = $self->readKeyword('pt', 'pc', 'in', 'bp', 'cm', 'mm', 'dd', 'cc', 'sp');
+    my $units = $STATE->lookupValue('UNITS');
+    $u = $self->readKeyword(keys %$units);
     if ($u) {
       $self->skip1Space;
       return $STATE->convertUnit($u); }
@@ -1009,4 +1011,3 @@ Public domain software, produced as part of work done by the
 United States Government & not subject to copyright in the US.
 
 =cut
-
