@@ -92,7 +92,7 @@ sub new {
     # it now can take a number of digits (%3p) for pages, BUT old version produces imgx-%3p.svg!!
     # So, simply use %p, let new version do 1,..,9,10,...
     # Recovery code below!
-    $$self{dvicmd}             = "dvisvgm --page=1- --bbox=1pt --mag=$mag --no-fonts -o imgx-${fmt}p";
+    $$self{dvicmd} = "dvisvgm --page=1- --bbox=1pt --scale=$$self{magnification} --no-fonts -o imgx-${fmt}p";
     $$self{dvicmd_output_name} = 'imgx-%02d.svg';
     $$self{dvicmd_output_type} = 'svg';
     $$self{frame_output}       = 0; }
@@ -182,8 +182,9 @@ sub cleanTeX {
     $style = $1; }
   $tex =~ s/^(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*//; # Trim leading spacing (especially negative!)
   $tex =~ s/(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*$//; # and trailing spacing
-  $tex =~ s/\%[^\n]*\n//gs;                                            # Strip comments
-  $tex = $style . ' ' . $tex if $style;                                # Put back the style, if any
+       # Strip comments, but watchout for \% (or more exactly, an ODD number of \)
+  $tex =~ s/(?<!\\)((?:\\\\)*)\%[^\n]*\n/$1/gs;
+  $tex = $style . ' ' . $tex if $style;    # Put back the style, if any
   return $tex; }
 
 #**********************************************************************
@@ -263,6 +264,7 @@ sub generateImages {
     if (!open($TEX, '>', $texfile)) {
       Error('I/O', $texfile, undef, "Cant write to '$texfile'", "Response was: $!");
       return $doc; }
+    binmode $TEX, ':encoding(UTF-8)';
     print $TEX $self->pre_preamble($doc);
     print $TEX "\\makeatletter\n";
     print $TEX $self->preamble($doc) . "\n";
@@ -361,28 +363,6 @@ sub generateImages {
   $doc->closeCache;    # If opened.
   return $doc; }
 
-# Get a list blah, blah...
-sub find_documentclass_and_packages {
-  my ($self, $doc) = @_;
-  my ($class, $classoptions, $oldstyle, @packages);
-  foreach my $pi ($doc->findnodes(".//processing-instruction('latexml')")) {
-    my $data  = $pi->textContent;
-    my $entry = {};
-    while ($data =~ s/\s*([\w\-\_]*)=([\"\'])(.*?)\2//) {
-      $$entry{$1} = $3; }
-    if ($$entry{class}) {
-      $class        = $$entry{class};
-      $classoptions = $$entry{options} || 'onecolumn';
-      $oldstyle     = $$entry{oldstyle}; }
-    elsif ($$entry{package}) {
-      push(@packages, [$$entry{package}, $$entry{options} || '']); }
-  }
-  if (!$class) {
-    Warn('expected', 'class', undef, "No document class found; using article");
-    $class = 'article'; }
-
-  return ([$class, $classoptions, $oldstyle], @packages); }
-
 #======================================================================
 # Generating & Processing the LaTeX source.
 #======================================================================
@@ -477,7 +457,7 @@ sub convert_image {
   my ($bg, $fg) = ($$self{background}, 'black');
 
   my $image = image_object(antialias => 'True', background => $bg, density => $$self{dpi});
-  my $err = $image->Read($$self{dvicmd_output_type} . ':' . $src);
+  my $err   = $image->Read($$self{dvicmd_output_type} . ':' . $src);
   if ($err) {
     Warn('imageprocessing', 'read', undef,
       "Image conversion failed to read '$src'",
@@ -523,7 +503,7 @@ sub DESTROY {
       my @empty_magick = grep { -z $_ } map { "$tmpdir/$_" } readdir($tmpdir_fh);
       closedir($tmpdir_fh);
       unlink $_ foreach @empty_magick;
-    } }
+  } }
   return; }
 
 #======================================================================
